@@ -81,7 +81,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         self.swift.register('GET', segment_bucket, swob.HTTPOk, {},
                             object_list)
         self.swift.register('HEAD', segment_bucket + '/object/X',
-                            swob.HTTPOk, {'x-object-meta-foo': 'bar'}, None)
+                            swob.HTTPOk, {'x-object-meta-foo': 'bar',
+                                          'content-type': 'baz/quux'}, None)
         self.swift.register('PUT', segment_bucket + '/object/X',
                             swob.HTTPCreated, {}, None)
         self.swift.register('DELETE', segment_bucket + '/object/X',
@@ -125,7 +126,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_bucket_upload_part(self):
         req = Request.blank('/bucket?partNumber=1&uploadId=x',
                             environ={'REQUEST_METHOD': 'PUT'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidRequest')
 
@@ -133,7 +135,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_object_multipart_uploads_list(self):
         req = Request.blank('/bucket/object?uploads',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidRequest')
 
@@ -141,7 +144,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_bucket_multipart_uploads_initiate(self):
         req = Request.blank('/bucket?uploads',
                             environ={'REQUEST_METHOD': 'POST'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidRequest')
 
@@ -149,7 +153,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_bucket_list_parts(self):
         req = Request.blank('/bucket?uploadId=x',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidRequest')
 
@@ -157,7 +162,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_bucket_multipart_uploads_abort(self):
         req = Request.blank('/bucket?uploadId=x',
                             environ={'REQUEST_METHOD': 'DELETE'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidRequest')
 
@@ -165,7 +171,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_bucket_multipart_uploads_complete(self):
         req = Request.blank('/bucket?uploadId=x',
                             environ={'REQUEST_METHOD': 'POST'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidRequest')
 
@@ -183,7 +190,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         query = '?uploads&' + query if query else '?uploads'
         req = Request.blank('/bucket/%s' % query,
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         return self.call_swift3(req)
 
     @s3acl
@@ -217,7 +225,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
 
         req = Request.blank('/bucket?uploads',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
 
         status, haeaders, body = self.call_swift3(req)
 
@@ -238,7 +247,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
                             swob.HTTPNotFound, {}, '')
         req = Request.blank('/bucket?uploads',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, haeaders, body = self.call_swift3(req)
         self.assertEquals(status.split()[0], '404')
         self.assertEquals(self._get_error_code(body), 'NoSuchBucket')
@@ -539,6 +549,7 @@ class TestSwift3MultiUpload(Swift3TestCase):
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization':
                                      'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(),
                                      'x-amz-meta-foo': 'bar'})
         status, headers, body = self.call_swift3(req)
         fromstring(body, 'InitiateMultipartUploadResult')
@@ -554,6 +565,7 @@ class TestSwift3MultiUpload(Swift3TestCase):
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization':
                                      'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(),
                                      'x-amz-acl': 'public-read',
                                      'x-amz-meta-foo': 'bar'})
         status, headers, body = self.call_swift3(req)
@@ -577,7 +589,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         req = Request.blank('/bucket/object?uploads',
                             environ={'REQUEST_METHOD': 'POST'},
                             headers={'Authorization':
-                                     'AWS test:tester:hmac'})
+                                     'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(status.split()[0], '404')
         self.assertEquals(self._get_error_code(body), 'NoSuchBucket')
@@ -587,7 +600,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         malformed_xml = 'malformed_XML'
         req = Request.blank('/bucket/object?uploadId=X',
                             environ={'REQUEST_METHOD': 'POST'},
-                            headers={'Authorization': 'AWS test:tester:hmac'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()},
                             body=malformed_xml)
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'MalformedXML')
@@ -595,7 +609,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         # without target bucket
         req = Request.blank('/nobucket/object?uploadId=X',
                             environ={'REQUEST_METHOD': 'POST'},
-                            headers={'Authorization': 'AWS test:tester:hmac'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(), },
                             body=xml)
         with patch('swift3.request.get_container_info',
                    lambda x, y: {'status': 404}):
@@ -607,7 +622,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_object_multipart_upload_complete(self):
         req = Request.blank('/bucket/object?uploadId=X',
                             environ={'REQUEST_METHOD': 'POST'},
-                            headers={'Authorization': 'AWS test:tester:hmac'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(), },
                             body=xml)
         status, headers, body = self.call_swift3(req)
         fromstring(body, 'CompleteMultipartUploadResult')
@@ -615,6 +631,7 @@ class TestSwift3MultiUpload(Swift3TestCase):
 
         _, _, headers = self.swift.calls_with_headers[-2]
         self.assertEquals(headers.get('X-Object-Meta-Foo'), 'bar')
+        self.assertEquals(headers.get('Content-Type'), 'baz/quux')
 
     @s3acl(s3acl_only=True)
     def test_object_multipart_upload_complete_s3acl(self):
@@ -624,11 +641,13 @@ class TestSwift3MultiUpload(Swift3TestCase):
         headers[sysmeta_header('object', 'tmpacl')] = \
             acl_headers.get(sysmeta_header('object', 'acl'))
         headers['X-Object-Meta-Foo'] = 'bar'
+        headers['Content-Type'] = 'baz/quux'
         self.swift.register('HEAD', '/v1/AUTH_test/bucket+segments/object/X',
                             swob.HTTPOk, headers, None)
         req = Request.blank('/bucket/object?uploadId=X',
                             environ={'REQUEST_METHOD': 'POST'},
-                            headers={'Authorization': 'AWS test:tester:hmac'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()},
                             body=xml)
         status, headers, body = self.call_swift3(req)
         fromstring(body, 'CompleteMultipartUploadResult')
@@ -636,6 +655,7 @@ class TestSwift3MultiUpload(Swift3TestCase):
 
         _, _, headers = self.swift.calls_with_headers[-2]
         self.assertEquals(headers.get('X-Object-Meta-Foo'), 'bar')
+        self.assertEquals(headers.get('Content-Type'), 'baz/quux')
         self.assertEquals(tostring(ACLPublicRead(Owner('test:tester',
                                                        'test:tester')).elem()),
                           tostring(decode_acl('object', headers).elem()))
@@ -644,14 +664,16 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_object_multipart_upload_abort_error(self):
         req = Request.blank('/bucket/object?uploadId=invalid',
                             environ={'REQUEST_METHOD': 'DELETE'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'NoSuchUpload')
 
         # without target bucket
         req = Request.blank('/nobucket/object?uploadId=X',
                             environ={'REQUEST_METHOD': 'DELETE'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         with patch('swift3.request.get_container_info',
                    lambda x, y: {'status': 404}):
             self.swift.register('HEAD', '/v1/AUTH_test/nobucket',
@@ -663,7 +685,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_object_multipart_upload_abort(self):
         req = Request.blank('/bucket/object?uploadId=X',
                             environ={'REQUEST_METHOD': 'DELETE'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(status.split()[0], '204')
 
@@ -673,7 +696,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         # without upload id
         req = Request.blank('/bucket/object?partNumber=1',
                             environ={'REQUEST_METHOD': 'PUT'},
-                            headers={'Authorization': 'AWS test:tester:hmac'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()},
                             body='part object')
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidArgument')
@@ -681,7 +705,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         # invalid part number
         req = Request.blank('/bucket/object?partNumber=invalid&uploadId=X',
                             environ={'REQUEST_METHOD': 'PUT'},
-                            headers={'Authorization': 'AWS test:tester:hmac'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()},
                             body='part object')
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidArgument')
@@ -689,7 +714,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         # part number must be > 0
         req = Request.blank('/bucket/object?partNumber=0&uploadId=X',
                             environ={'REQUEST_METHOD': 'PUT'},
-                            headers={'Authorization': 'AWS test:tester:hmac'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()},
                             body='part object')
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidArgument')
@@ -697,7 +723,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         # part number must be < 1000
         req = Request.blank('/bucket/object?partNumber=1001&uploadId=X',
                             environ={'REQUEST_METHOD': 'PUT'},
-                            headers={'Authorization': 'AWS test:tester:hmac'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()},
                             body='part object')
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidArgument')
@@ -705,7 +732,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         # without target bucket
         req = Request.blank('/nobucket/object?partNumber=1&uploadId=X',
                             environ={'REQUEST_METHOD': 'PUT'},
-                            headers={'Authorization': 'AWS test:tester:hmac'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()},
                             body='part object')
         with patch('swift3.request.get_container_info',
                    lambda x, y: {'status': 404}):
@@ -718,7 +746,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_object_upload_part(self):
         req = Request.blank('/bucket/object?partNumber=1&uploadId=X',
                             environ={'REQUEST_METHOD': 'PUT'},
-                            headers={'Authorization': 'AWS test:tester:hmac'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()},
                             body='part object')
         status, headers, body = self.call_swift3(req)
         self.assertEquals(status.split()[0], '200')
@@ -727,14 +756,16 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_object_list_parts_error(self):
         req = Request.blank('/bucket/object?uploadId=invalid',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'NoSuchUpload')
 
         # without target bucket
         req = Request.blank('/nobucket/object?uploadId=X',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         with patch('swift3.request.get_container_info',
                    lambda x, y: {'status': 404}):
             self.swift.register('HEAD', '/v1/AUTH_test/nobucket',
@@ -746,7 +777,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_object_list_parts(self):
         req = Request.blank('/bucket/object?uploadId=X',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         elem = fromstring(body, 'ListPartsResult')
         self.assertEquals(elem.find('Bucket').text, 'bucket')
@@ -778,7 +810,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
                             swob.HTTPOk, {}, None)
         req = Request.blank('/bucket/object@@?uploadId=X&encoding-type=url',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         elem = fromstring(body, 'ListPartsResult')
         self.assertEquals(elem.find('Key').text, quote('object@@'))
@@ -790,7 +823,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
                             swob.HTTPOk, {}, None)
         req = Request.blank('/bucket/object@@?uploadId=X',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         elem = fromstring(body, 'ListPartsResult')
         self.assertEquals(elem.find('Key').text, 'object@@')
@@ -799,14 +833,16 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_object_list_parts_encoding_type_error(self):
         req = Request.blank('/bucket/object?uploadId=X&encoding-type=xml',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidArgument')
 
     def test_object_list_parts_max_parts(self):
         req = Request.blank('/bucket/object?uploadId=X&max-parts=1',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         elem = fromstring(body, 'ListPartsResult')
         self.assertEquals(elem.find('IsTruncated').text, 'true')
@@ -816,14 +852,16 @@ class TestSwift3MultiUpload(Swift3TestCase):
     def test_object_list_parts_str_max_parts(self):
         req = Request.blank('/bucket/object?uploadId=X&max-parts=invalid',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidArgument')
 
     def test_object_list_parts_negative_max_parts(self):
         req = Request.blank('/bucket/object?uploadId=X&max-parts=-1',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidArgument')
 
@@ -831,7 +869,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         req = Request.blank('/bucket/object?uploadId=X&max-parts=%d' %
                             (CONF.max_parts_listing + 1),
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         elem = fromstring(body, 'ListPartsResult')
         self.assertEquals(elem.find('Bucket').text, 'bucket')
@@ -860,7 +899,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         req = Request.blank('/bucket/object?uploadId=X&max-parts=%d' %
                             (MAX_32BIT_INT + 1),
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidArgument')
 
@@ -868,7 +908,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         req = Request.blank('/bucket/object?uploadId=X&'
                             'part-number-marker=1',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         elem = fromstring(body, 'ListPartsResult')
         self.assertEquals(len(elem.findall('Part')), 1)
@@ -880,7 +921,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         req = Request.blank('/bucket/object?uploadId=X&part-number-marker='
                             'invalid',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidArgument')
 
@@ -888,7 +930,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         req = Request.blank('/bucket/object?uploadId=X&part-number-marker='
                             '-1',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidArgument')
 
@@ -897,7 +940,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         req = Request.blank('/bucket/object?uploadId=X&'
                             'part-number-marker=%s' % part_number_marker,
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         elem = fromstring(body, 'ListPartsResult')
         self.assertEquals(len(elem.findall('Part')), 0)
@@ -909,14 +953,16 @@ class TestSwift3MultiUpload(Swift3TestCase):
         req = Request.blank('/bucket/object?uploadId=X&part-number-marker='
                             '%s' % ((MAX_32BIT_INT + 1)),
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         self.assertEquals(self._get_error_code(body), 'InvalidArgument')
 
     def test_object_list_parts_same_max_marts_as_objects_num(self):
         req = Request.blank('/bucket/object?uploadId=X&max-parts=2',
                             environ={'REQUEST_METHOD': 'GET'},
-                            headers={'Authorization': 'AWS test:tester:hmac'})
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
         status, headers, body = self.call_swift3(req)
         elem = fromstring(body, 'ListPartsResult')
         self.assertEquals(len(elem.findall('Part')), 2)
@@ -926,7 +972,8 @@ class TestSwift3MultiUpload(Swift3TestCase):
         path = '/bucket%s' % ('/object' + query if hasObj else query)
         req = Request.blank(path,
                             environ={'REQUEST_METHOD': method},
-                            headers={'Authorization': 'AWS %s:hmac' % account},
+                            headers={'Authorization': 'AWS %s:hmac' % account,
+                                     'Date': self.get_date_header()},
                             body=body)
         return self.call_swift3(req)
 
@@ -1062,17 +1109,19 @@ class TestSwift3MultiUpload(Swift3TestCase):
                             head_resp, src_o_headers, None)
 
         put_headers = {'Authorization': 'AWS %s:hmac' % account,
+                       'Date': self.get_date_header(),
                        'X-Amz-Copy-Source': src_path}
         put_headers.update(put_header)
         req = Request.blank(
             '/bucket/object?partNumber=1&uploadId=X',
-            environ={'REQUEST_METHOD': 'PUT'},
+            environ={'REQUEST_METHOD': 'PUT',
+                     'HTTP_X_TIMESTAMP': '1396353600.000000'},
             headers=put_headers)
         return self.call_swift3(req)
 
     @s3acl
     def test_upload_part_copy(self):
-        last_modified = '2014-04-01T12:00:00'
+        last_modified = '2014-04-01T12:00:00.000Z'
         status, headers, body = \
             self._test_copy_for_s3acl('test:tester')
         self.assertEquals(status.split()[0], '200')
